/*
 * Copyright 2016-2020 Crown Copyright
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package uk.gov.gchq.gaffer.commonutil.iterable;

import java.io.Closeable;
import java.util.Iterator;

/**
 * A {@code CloseableIterator} is an {@link java.util.Iterator} which must provide an implementation
 * of the {@link java.io.Closeable#close()} method.
 *
 * @param <T> the type of items in the iterator.
 */
public interface CloseableIterator<T> extends Iterator<T>, Closeable {
    @Override
    void close();
}
